#!/bin/zsh

# Copyright 2014, 2016, 2017 Paul Hänsch
#
# This file is part of Confetti.
# 
# Confetti is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Confetti is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Confetti.  If not, see <http://www.gnu.org/licenses/>. 

[ -z "${_GET[order]}" ] && _GET[order]=DOW

listcards() {
  ls -1 ${_DATA}/vcard/*vcf 2>/dev/null |while read file; do
    fn=$(sed -rn 's:^N(;.+)*\:([^;]*;){1} *([^;]*).*$:\3:p' "$file")
    echo "$fn\t$file"
  done |sort |sed -r 's:^.*\t(.*/)([^/]+)$:\2:'
}


listcourses() {
  case "${_GET[filtertype]}" in
    *) ls -1 ${_DATA}/ical/*ics 2>/dev/null
      ;;
  esac |case "${_GET[order]}" in
    DOW)
      while read file; do
        icstime="$(sed -rn 's:^DTSTART\:(TZID=.*\:)?([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z?\r$:\2-\3-\4 \5\:\6\:\7:p' "$file")"
        echo "$(date -d "$icstime" "+%u %H%M%S")\t$file"
      done
      ;;
    TOD)
      while read file; do
        icstime="$(sed -rn 's:^DTSTART\:(TZID=.*\:)?([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z?\r$:\2-\3-\4 \5\:\6\:\7:p' "$file")"
        echo "$(date -d "$icstime" +%H%M%S)\t$file"
      done
      ;;
    *)
      sed -r 's:^.*$:x\t&:'
      ;;
  esac |sort |sed -r 's:^.*\t(.*/)([^/]+)$:\2:'
}

list_attendance() {
  id="$1"
  sed -rn 's:'$id'\t(.+)$:\1:p' "$_DATA/mappings/attendance" |while read each; do
    card="$_DATA/vcard/$each"
    if [ -r "$card" ]; then
      aname="$(sed -rn '
          /^N(;.+)*:/{
          h;
          s;^N(\;.+)*:([^\;]*\;){3} *([^\;]*).*$;\3;p; g;
          s;^N(\;.+)*:([^\;]*\;){1} *([^\;]*).*$;\3;p; g;
          s;^N(\;.+)*:([^\;]*\;){2} *([^\;]*).*$;\3;p; g;
          s;^N(\;.+)*:([^\;]*\;){0} *([^\;]*).*$;\3;p; g;
          s;^N(\;.+)*:([^\;]*\;){4} *([^\;]*).*$;\3;p; g;
          }' "$card" \
        | sed -r ':X;N;$!bX; s;([\;\n\r,]| )+; ;g;; s;^ +| +$;;g;'
      )"
      fname="$(sed -rn 's;^FN(\;.+)*:(.+)\r?$;\2;g; s;([\;\n,]| )+; ;g;; s;^ +| +$;;g;' "$card")"
      nname="$(sed -rn 's;^NICKNAME(\;.+)*:(.+)\r?$;\2;g; s;([\;\n,]| )+; ;g;; s;^ +| +$;;g;' "$card")"
      byear="$(sed -rn 's:^BDAY(\;.+)*\:([0-9]{4})(-[0-9][0-9]){2}.*$:\2:p' "$_DATA/vcard/$each")"

      printf '%s %s (*%04i)\n' "$each" "${aname:-${fname:-${nname}}}" "$byear"
    fi
  done
}

course_mail() {
  id="$1"
  coursemail=""
  sed -rn 's:'$id'\t(.+)$:\1:p' "$_DATA/mappings/attendance" |while read each; do
    coursemail="$coursemail$(sed -rn 's:^EMAIL(;.+)*\:(.+)\r$:\2,:p' "$_DATA/vcard/$each")"
  done
  echo "$coursemail"
}

ics_parse() {
  unset key
  sed -r ':X;N;$!bX; s;\r\n[ \t];;g; s;\r\n;\n;g;' "$1" \
  | sed -rn '
    # === turn property names to upper case, strip group names ===
    h; s;^([^;:]+);;;
    x; s;^([^;:\.]+\.)?([^;:]+).*$;\2;;
    y;abcdefghijklmnopqrstuvwxyz;ABCDEFGHIJKLMNOPQRSTUVWXYZ;
    G; s;\n;;;

    /^([A-Z0-9-]+)((\;[A-z0-9-]+=?|\;[A-z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):(.*)$/{
      h;
      s;^([A-Z0-9-]+)((\;[A-z0-9-]+=?|\;[A-z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):(.*)$;key='\''\1'\'';;
      H; g;

      s;^([A-Z0-9-]+)((\;[A-z0-9-]+=?|\;[A-z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):([^\n]*)\n.*$;\2;;
      s;'\'';'\'\\\\\'\'';g
      s;\;([A-Z0-9-]+)(=|=(([^\;,:"]+|"[^"]+")(,[^\;,:"]+|,"[^"]+")*))?;\ntag[\1]='\''\3'\'';g
      s;^\n+;;; s;\n+$;;;
      /^.+$/H; g;

      s;^([A-Z0-9-]+)((\;[A-z0-9-]+=?|\;[A-z0-9-]+=([^;,:"]+|"[^"]+")(,[^;,:"]+|,"[^"]+")*)*):([^\n]*)\n.*$;\6;;
      # :X s;((^|[^\\])(\\\\)*)[;,];\1\n;g; s;((^|[^\\])(\\\\)*)\\([;,]);\4;g; tX;
      s;'\'';'\'\\\\\'\'';g
      s;^.*$;value='\''&'\'';
      H; g;

      s;^[^\n]*[\n ]+;;
      s;\n+$;;;

      p;
    }
    ' \
  | while read -r line; do
    declare -A tag
    case "$line" in
      value*) eval "$line";;
      tag*)   eval "$line";;
      key*)
        if [ -z "$key" ]; then
          eval "$line"
        else
          values[$key]="$value"
          for t in ${(k)tag}; do
            values[${key}_$t]="$tag[$t]"
          done
          eval "$line"
          if [ -n "$values[$key]" ]; then
            n=0
            while [ -n "$values[$key$n]" ]; do n=$(($n + 1)); done
            key=$key$n
          fi
          unset value
          unset tag
        fi
      ;;
    esac
  done
}

view_course() {  #Parameter: Calendarfile
  id="$1"
  calendarfile="$_DATA/ical/${id}"
  cachefile="$_DATA/cache/${id}.cache"
  unset key
  if [ "$cachefile" -nt "$calendarfile" ]; then
    cat "$cachefile"
  else
    declare -A values
    ics_parse "$calendarfile"
    . ${_EXEC}/templates/view_course.sh |tee "$cachefile"
  fi
}

edit_course() {  #Parameter: Calendarfile
  id="$1"
  calendarfile="$_DATA/ical/$id"
  tempfile="$_DATA/temp/$id"
  [ -f "$tempfile" ] && calendarfile="$tempfile"
  unset key

  declare -A values
  ics_parse "$calendarfile"
  . ${_EXEC}/templates/edit_course.sh
}
